/********************************************************/
/*                                                      */
/*               OO7 Benchmark                          */
/*                                                      */
/*              COPYRIGHT (C) 1993                      */
/*                                                      */
/*                Michael J. Carey 		        */
/*                David J. DeWitt 		        */
/*                Jeffrey Naughton 		        */
/*               Madison, WI U.S.A.                     */
/*                                                      */
/*	         ALL RIGHTS RESERVED                    */
/*                                                      */
/********************************************************/

#include <libc.h>
#include <stdio.h>
#include <sys/resource.h>
#include <E/collection.h>

/* new */
#include <stdlib.h> 

#include "oo7.h"
#include "globals.h"
#include "GenParams.h"
#include "VarParams.h"
#include "IntBtree.h"
#include "StringBtree.h"
#include "stats.h"
#include "stats_server.h"
#include "baidlist.h"

int overFlowCnt;
int NoLogging = 1;
// this should be a command line flag.
// this should also be integerated into the E runtime.
// well maybe.

// the following is diagnostic code, not part of real run.
int TestAccess = 1;
// instrumentation function.
extern "C" void E_SetLastPage(dbvoid *cp);


//////////////////////////////////////////////////////////////////
//
// Global Variables, etc., for Benchmark Database Generation
//
//////////////////////////////////////////////////////////////////

struct rusage startUsage, endUsage;
struct timeval startWallTime;
struct timeval endWallTime;
struct timezone ignoreTimeZone;

extern SetParams(char* configFileName);

extern double ComputeWallClockTime(struct timeval *startWallTime, 
	           struct timeval *endWallTime);

extern double ComputeUserTime(struct rusage *startUsage, 
	           struct rusage *endUsage);

extern double ComputeSystemTime(struct rusage *startUsage, 
	           struct rusage *endUsage);

int	nextAtomicId=1;
int	nextCompositeId=1;
int	nextComplexAssemblyId=1;
int	nextBaseAssemblyId=1;
int	nextModuleId=1;
int	debugMode = FALSE;

char    *types[NumTypes] = {
		"type000", "type001", "type002", "type003", "type004",
		"type005", "type006", "type007", "type008", "type009"
	};

extern void createDummy(int);

// data structures to keep track of which composite parts each base
// assemblies uses.  This is done so that we can generate the base
// assemblies first and then the composite parts.  Why bother?
// In particular, if you did the composite parts first and the base
// assemblies, there would be no need for these data structures.
// This second approach worked fine for the small and medium databases
// but not for the large as the program seeked all over the disk.

BAIdList* private_cp;
BAIdList* shared_cp;

//////////////////////////////////////////////////////////////////
//
// Do Benchmark Data Generation
//
//////////////////////////////////////////////////////////////////

void gendb()
{
        int 		partDate;
        int 		id, typeNo, docLen;
	char 		moduleName[40];
	Module*		moduleH;
	CompositePart*	cp;

	// create all the indices needed
	AtomicPartIdx = new (CompPartSet) IntBtIndex;
	CompPartIdx = new (CompPartSet) IntBtIndex;
	DocumentIdx = new (DocumentSet) StringBtIndex;
	DocumentIdIdx = new (DocumentSet) IntBtIndex;
	ModuleIdx = new (ModuleSet) StringBtIndex;
	BaseAssemblyIdx = new (ModuleSet) IntBtIndex;

	printf("TotalCompParts = %d\n",TotalCompParts);

	shared_cp = new BAIdList[TotalCompParts+1];
	private_cp = new BAIdList[TotalCompParts+1];

	// First generate the desired number of modules
	while (nextModuleId <= TotalModules) 
	{
	    int id = nextModuleId++;
            sprintf(moduleName, "Module %08d", id);
	    moduleH = new (ModuleSet) Module (id);
	    ModuleIdx->insert(moduleName, moduleH);  // add module to index
	    AllModules.add(moduleH); // add module to extent
	    E_CommitTransaction();
	    E_BeginTransaction();
	    if (NoLogging)
		sm_SetLogLevel(LOG_SPACE, 0, 0);
	}

	printf("generated all the modules, now generated the comp parts\n");
	// now generate the composite parts
	while (nextCompositeId <= TotalCompParts) 
	{
	    id = nextCompositeId++;
	    cp = new (CompPartSet) CompositePart(id);
	    CompPartIdx->insert(id, cp);  // add composite part to its index
	    AllCompParts.add(cp);  // add composite part to its extent 
	    if (TotalCompParts >= 100)
	    {
	    if ((nextCompositeId % 25)==0)
	    {
		printf("progress : nextCompositeId=%d\n", nextCompositeId);
		if (TestAccess)
			E_SetLastPage(cp);
		if ((nextCompositeId%100)==0)
		{
			E_CommitTransaction();
			E_BeginTransaction();
			if (NoLogging)
			    sm_SetLogLevel(LOG_SPACE, 0, 0);
			printf("intermediate commmit done\n");
		}
	    }
            }
        }
	E_CommitTransaction();

	// Print out some useful information about what happened

	printf("=== DONE CREATING DATABASE, TOTALS WERE ===\n");
	printf("    # atomic parts\t\t%d\n", nextAtomicId-1);
	printf("    # composite parts\t\t%d\n", nextCompositeId-1);
	printf("    # complex assemblies\t%d\n", nextComplexAssemblyId-1);
	printf("    # base assemblies\t\t%d\n", nextBaseAssemblyId-1);
	printf("    # modules\t\t\t%d\n", nextModuleId-1);
	printf("    # overFlowCnt\t\t\t%d\n", overFlowCnt);

	//////////////////////////////////////////////////////////////////
	//
	// Shutdown DB
	//
	//////////////////////////////////////////////////////////////////

        // compute and report wall clock time
        gettimeofday(&endWallTime, &ignoreTimeZone);
        printf("Wall-Clock time to generate database: %f seconds.\n", 
	             ComputeWallClockTime(&startWallTime, &endWallTime));

	getrusage(RUSAGE_SELF, &endUsage);
	fprintf(stdout, "(CPU: %f seconds.)\n", 
	                ComputeUserTime(&startUsage, &endUsage) +
			ComputeSystemTime(&startUsage, &endUsage));

}


main(int argc, char** argv)
{
	ProcessStats    totalTime;
	ServerStats     totalSrvTime;
	char*		purgeVar;

	if (argc < 2) 
	{
	      fprintf(stderr, "Usage: %s <configFileName> [-d]\n", argv[0]);
	      exit(1);
	}
	// See if debug mode is desired
	if ((argc == 3) && (strcmp(argv[2], "-d") == 0)) debugMode = TRUE;
	else  debugMode = FALSE;
	
	SetParams(argv[1]);

        // get wall clock time
        gettimeofday(&startWallTime, &ignoreTimeZone);

	// get starting usage values.
	getrusage(RUSAGE_SELF, &startUsage);

	totalTime.Start();
	totalSrvTime.Start();
	E_BeginTransaction();
	if (NoLogging)
	    sm_SetLogLevel(LOG_SPACE, 0, 0);
	gendb(); // generate the database

	totalTime.Stop();
	totalSrvTime.Stop();
	fprintf(stdout, "Total stats (client,server):\n");
	totalTime.PrintStatsHeader(stdout);
	totalTime.PrintStats(stdout, "TotalCli");
	totalSrvTime.PrintStats(stdout, "TotalSrv");

        // create 60,000 1000 byte dummy objects
	purgeVar = getenv ("PURGE");
	if (purgeVar != NULL)
	{
		if (atoi(purgeVar) == 1)
		{
			E_BeginTransaction();
			if (NoLogging)
	    			sm_SetLogLevel(LOG_SPACE, 0, 0);
        		printf("creating 60000 dummy 1,000 byte objects\n");
			createDummy (60000);
			E_CommitTransaction();
		}
		else printf("not creating purge collection\n");
	}
	else printf("not creating purge collection\n");
}
